@@ -7,7 +7,7 @@
 ;; Keywords: tools, accessibility
 ;; URL: https://gitlab.com/michelangelo-rodriguez/greader
 
-;; Version: 0.11.18
+;; Version: 0.12.1
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@
 (require 'seq)
 (require 'view)
 (defvar-local greader-timer-flag nil)
-
+(require 'find-func)
 (defvar greader-auto-tired-timer nil)
 (defvar greader-auto-tired-end-timer)
 (defvar greader-last-point nil)
@@ -62,6 +62,8 @@
 (defvar greader-backend-action #'greader--default-action)
 (defvar greader-status 'paused)
 (defvar greader-synth-process nil)
+(defvar-local greader-process-directory (file-name-directory (find-library-name "greader"))
+  "The directory where execution should start.")
 
 
 (define-obsolete-variable-alias 'greader-before-get-sentence-functions
@@ -124,9 +126,12 @@ If all the functions in the hook return nil, this function return nil."
 
 (defcustom
   greader-backends
-  '(greader-espeak greader-speechd)
-  "A list of functions that are back-ends for greader."
-  :tag "greader back-ends"
+  '(greader-espeak greader-speechd greader-mac greader-piper)
+  "A list of functions that are back-ends for greader.
+If you have already customized this variable and saved it, and if a
+new back-end was released or you have created yours, you should add
+those new back-ends manually,  so if you want to keep yourself
+updated on new back-ends, please check the pproject's repository."
   :type '(repeat function))
 
 (defcustom
@@ -233,7 +238,8 @@ if set to t, when you call function `greader-read', that function sets a
   is set at register position then reading starts from there."
   :type 'boolean
   :tag "use register")
-
+(defvar greader-reading-mode)
+(defvar greader-mode)
 (defun greader-set-reading-keymap ()
   "Set greader's keymap when reading."
   (setq greader-mode nil)
@@ -439,10 +445,12 @@ available backends."
       (greader-debug "greader-read-asynchronous entered\n"))
   (run-hooks 'greader-before-read-hook)
   (greader-build-args)
+  (setq txt (substring-no-properties txt))
   (if (and txt (greader-sentence-needs-dehyphenation txt))
       (setq txt (greader-dehyphenate txt)))
   (let* ((txt (concat " " txt))
-         (backend (append greader-backend `(,txt))))
+         (backend (append greader-backend `(,txt)))
+	 (default-directory greader-process-directory))
     (and (stringp txt)
 	 (setq-local greader-synth-process (make-process
 				            :name "greader-backend"
@@ -527,7 +535,12 @@ Optional argument EVENT ."
 	'not-implemented))
       (setq arg (greader-call-backend 'extra))
       (setq args (append `(,arg) args))))
-    (setq greader-backend (append `(,greader-backend) args))))
+    (catch 'deleted
+      (dolist (argument args)
+	(when (equal argument 'not-implemented)
+	  (setq args (delete argument args))
+	  (throw 'deleted t))))
+      (setq greader-backend (append `(,greader-backend) args))))
 
 (defun greader-reset ()
   "Reset greader."
@@ -540,7 +553,9 @@ Argument EVENT ."
   (if greader-debug
       (greader-debug (format "greader-next-action: %s" event)))
   (run-hooks 'greader-after-read-hook)
-  (funcall greader-move-to-next-chunk)
+  (cond
+   ((equal event "finished\n")
+    (funcall greader-move-to-next-chunk)))
   (greader-read))
 
 (defun greader-response-for-dissociate (&optional _prompt)
@@ -1209,16 +1224,18 @@ Hook for `after-save-hook'."
 If nil, you can not use `greader-compile-at-point'."
   :tag "greader compile default dictionary source file"
   :type 'string)
-
+(defcustom greader-compile-history-delete-duplicates t
+  "If enabled, duplicates in the history will not be added."
+  :type 'boolean)
 (defvar greader-espeak-language)
 (defun greader-compile-at-point (&optional src dst)
   "Add a word to the espeak dictionary definition.
 When called interactively and point is on a word, this function asks
-  for the definition, assuming you want to define current word.
+for the definition, assuming you want to define current word.
 If you want to be asked about the word to define, call this command
-  with prefix.
+with prefix.
 When called from a function, you should specify SRC and DST, even if
-  SRC and DST are declared as optional."
+SRC and DST are declared as optional."
   (interactive "P")
   (unless greader-compile-default-source
     (error
@@ -1234,8 +1251,9 @@ When called from a function, you should specify SRC and DST, even if
     (if (equal src "")
 	(setq src (thing-at-point 'word t))))
   (unless dst
-    (setq dst (read-string (concat "Redefine " src " to: ") nil
-			   'greader-compile-history)))
+    (let ((history-delete-duplicates  greader-compile-history-delete-duplicates))
+      (setq dst (read-string (concat "Redefine " src " to: ") nil
+			   'greader-compile-history))))
 
   (let ((lang-file
 	 (if (string-prefix-p "/" greader-compile-default-source)
@@ -1743,5 +1761,85 @@ If `greader-region-mode' is enabled, restart will behave accordingly."
     (remove-hook 'greader-before-finish-functions
 		 #'greader-study-restart t)))
 
+;; Estimated reading time calculation.
+(defvar-local greader--buffer-words nil
+  "Total buffer words.")
+(defun greader--estimated-reading-time (&optional start end)
+  "Calculate the estimated reading time between START and END, based
+on your reading speed in WPM (words per minute).
+The selected back-end must support `rate' command.
+If START or END is not provided, consider the entire buffer."
+  (if-let* ((wpm (and (numberp (greader-get-rate))
+                      (greader-get-rate))))
+      (progn
+	(setq greader--buffer-words (if greader--buffer-words
+					greader--buffer-words (count-words (or start (point-min))
+									   (or end (point-max)))))
+	(let* ((total-minutes (/ (* 1.0 greader--buffer-words) wpm))
+               (total-seconds (round (* total-minutes 60)))
+               (hours (/ total-seconds 3600))
+             (minutes (/ (% total-seconds 3600) 60))
+             (seconds (% total-seconds 60)))
+          ;; (message "%02d:%02d:%02d" hours minutes seconds)
+	  (list (propertize (format "%02d:%02d:%02d" hours minutes
+				    seconds) 'greader-estimated-time t)
+		`(,hours ,minutes ,seconds))))
+    nil))
+
+(defun greader--estimated-reading-time-remove ()
+  "Remove estimated-time banner from the mode line."
+  (catch 'banner-found
+    (dolist (elt mode-line-misc-info)
+      (if (listp elt)
+	  (let ((line (cadr elt)))
+	    (and
+	     (equal (car elt) :propertize)
+	     (text-property-any 0 (- (length line) 1)
+				'greader-estimated-time t line)
+	     (setq-local mode-line-misc-info (delete elt
+					       mode-line-misc-info))))
+	(throw 'banner-found t))
+      nil)))
+(declare-function dtk-speak nil)
+(defun greader-estimated-time-update ()
+  "update of estimated reading time.
+new data is stored in the mode-line.
+The calculation is made from point to end of the buffer.
+If the region is active, use it."
+  (interactive)
+  (setq greader--buffer-words nil)
+  (greader--estimated-reading-time-remove)
+  (let ((region-beginning
+	 (if (region-active-p) (region-beginning) (point)))
+	(region-end
+	 (if (region-active-p) (region-end) (point-max))))
+    (add-to-list 'mode-line-misc-info (list :propertize (car (greader--estimated-reading-time
+							 region-beginning region-end))
+						   'greader-estimated-time
+						   t))
+      (when (and (featurep 'emacspeak) (called-interactively-p 'any))
+	(dtk-speak (car (greader--estimated-reading-time)))))
+  (force-mode-line-update))
+
+;;;###autoload
+(define-minor-mode greader-estimated-time-mode
+  "keep mode line updated in respect of remaining reading time.
+This mode updates the mode line on certain events, such as when pass
+to the next sentence, or when you stop the reading."
+  :lighter " "
+  (if greader-estimated-time-mode
+      (progn
+	(add-hook 'greader-before-read-hook
+		  #'greader-estimated-time-update)
+	(add-hook 'greader-after-stop-hook
+		  #'greader-estimated-time-update))
+    (remove-hook 'greader-before-read-hook 'greader-estimated-time-update)
+    (remove-hook 'greader-after-stop-hook
+		 'greader-estimated-time-update)
+    (greader--estimated-reading-time-remove))
+  (force-mode-line-update))
+
+    
+
 (provide 'greader)
 ;;; greader.el ends here
